Class: Oj::Parser
- Inherits:
-
Object
- Object
- Oj::Parser
- Defined in:
- ext/oj/parser.c,
ext/oj/parser.c
Overview
A reusable parser that makes use of named delegates to determine the handling of parsed data. Delegates are available for validation, a callback parser (SAJ), and a usual delegate that builds Ruby objects as parsing proceeds.
This parser is considerably faster than the older Oj.parse call and isolates options to just the parser so that other parts of the code are not forced to use the same options.
Class Method Summary collapse
-
.new(mode = nil) ⇒ Object
Creates a new Parser with the specified mode.
- .safe(*args) ⇒ Object
-
.saj ⇒ Object
Returns the default SAJ parser.
-
.usual ⇒ Object
Returns the default usual parser.
-
.validate ⇒ Object
Returns the default validate parser.
Instance Method Summary collapse
-
#file(filename) ⇒ Object
Parse a JSON file.
-
#just_one ⇒ Object
Returns the current state of the just_one [Boolean] option.
-
#just_one=(value) ⇒ Object
Sets the just_one option which limits the parsing of a string or or stream to a single JSON element.
-
#load(reader) ⇒ Object
Parse a JSON stream.
-
#method_missing(value) ⇒ Object
Methods not handled by the parser are passed to the delegate.
-
#parse(json) ⇒ Object
Parse a JSON string.
Dynamic Method Handling
This class handles dynamic methods through the method_missing method
#method_missing(value) ⇒ Object
Methods not handled by the parser are passed to the delegate. The methods supported by delegate are:
-
:validate
-
no options
-
-
:saj
-
cache_keys is a flag indicating hash keys should be cached.
-
cache_strings is a positive integer less than 35. Strings shorter than that length are cached.
-
handler is the SAJ handler
-
-
:usual
-
cache_keys is a flag indicating hash keys should be cached.
-
cache_strings is a positive integer less than 35. Strings shorter than that length are cached.
-
cache_expunge dictates when the cache will be expunged where 0 never expunges, 1 expunges slowly, 2 expunges faster, and 3 or higher expunges agressively.
-
capacity is the capacity of the parser’s internal stack. The parser grows automatically but can be updated directly with this call.
-
create_id if non-nil is the key that is used to specify the type of object to create when parsing. Parsed JSON objects that include the specified element use the element value as the name of the class to create an object from instead of a Hash.
-
decimal is the approach to how decimals are parsed. If :auto then the decimals with significant digits are 16 or less are Floats and long ones are BigDecimal. :ruby uses a call to Ruby to convert a string to a Float. :float always generates a Float. :bigdecimal always results in a BigDecimal.
-
ignore_json_create is a flag that when set the class json_create method is ignored on parsing in favor of creating an instance and populating directly.
-
missing_class is an indicator that determines how unknown class names are handled. Valid values are :auto which creates any missing classes on parse, :ignore which ignores and continues as a Hash (default), and :raise which raises an exception if the class is not found.
-
omit_null is a flag that if true then null values in a map or object are omitted from the resulting Hash or Object.
-
symbol_keys is a flag that indicates Hash keys should be parsed to Symbols versus Strings.
-
1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 1115 |
# File 'ext/oj/parser.c', line 1092
static VALUE parser_missing(int argc, VALUE *argv, VALUE self) {
ojParser p;
const char *key = NULL;
volatile VALUE rkey = *argv;
volatile VALUE rv = Qnil;
TypedData_Get_Struct(self, struct _ojParser, &oj_parser_type, p);
#if HAVE_RB_EXT_RACTOR_SAFE
// This doesn't seem to do anything.
rb_ext_ractor_safe(true);
#endif
switch (rb_type(rkey)) {
case RUBY_T_SYMBOL:
rkey = rb_sym2str(rkey);
// fall through
case RUBY_T_STRING: key = StringValuePtr(rkey); break;
default: rb_raise(rb_eArgError, "option method must be a symbol or string");
}
if (1 < argc) {
rv = argv[1];
}
return p->option(p, key, rv);
}
|
Class Method Details
.new(mode = nil) ⇒ Object
Creates a new Parser with the specified mode. If no mode is provided validation is assumed. Optional arguments can be provided that match the mode. For example with the :usual mode the call might look like Oj::Parser.new(:usual, cache_keys: true).
975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 |
# File 'ext/oj/parser.c', line 975
static VALUE parser_new(int argc, VALUE *argv, VALUE self) {
ojParser p = OJ_R_ALLOC(struct _ojParser);
#if HAVE_RB_EXT_RACTOR_SAFE
// This doesn't seem to do anything.
rb_ext_ractor_safe(true);
#endif
memset(p, 0, sizeof(struct _ojParser));
buf_init(&p->key);
buf_init(&p->buf);
p->map = value_map;
if (argc < 1) {
oj_set_parser_validator(p);
} else {
VALUE mode = argv[0];
if (Qnil == mode) {
oj_set_parser_validator(p);
} else {
const char *ms = NULL;
switch (rb_type(mode)) {
case RUBY_T_SYMBOL:
mode = rb_sym2str(mode);
// fall through
case RUBY_T_STRING: ms = RSTRING_PTR(mode); break;
default: rb_raise(rb_eArgError, "mode must be :validate, :usual, :saj, or :object");
}
if (0 == strcmp("usual", ms) || 0 == strcmp("standard", ms) || 0 == strcmp("strict", ms) ||
0 == strcmp("compat", ms)) {
oj_set_parser_usual(p);
} else if (0 == strcmp("object", ms)) {
// TBD
} else if (0 == strcmp("saj", ms)) {
oj_set_parser_saj(p);
} else if (0 == strcmp("validate", ms)) {
oj_set_parser_validator(p);
} else if (0 == strcmp("debug", ms)) {
oj_set_parser_debug(p);
} else {
rb_raise(rb_eArgError, "mode must be :validate, :usual, :saj, or :object");
}
}
if (1 < argc) {
VALUE ropts = argv[1];
Check_Type(ropts, T_HASH);
rb_hash_foreach(ropts, opt_cb, (VALUE)p);
}
}
return TypedData_Wrap_Struct(parser_class, &oj_parser_type, p);
}
|
.safe(*args) ⇒ Object
1364 1365 1366 1367 1368 1369 1370 1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 1381 1382 1383 1384 |
# File 'ext/oj/parser.c', line 1364
static VALUE parser_safe(int argc, VALUE *argv, VALUE self) {
VALUE options;
if (1 == argc) {
options = argv[0];
Check_Type(options, T_HASH);
} else {
options = rb_hash_new();
}
ojParser p = OJ_R_ALLOC(struct _ojParser);
memset(p, 0, sizeof(struct _ojParser));
buf_init(&p->key);
buf_init(&p->buf);
p->map = value_map;
oj_set_parser_safe(p, options);
return TypedData_Wrap_Struct(parser_class, &oj_parser_type, p);
}
|
.saj ⇒ Object
Returns the default SAJ parser. Note the default SAJ parser can not be used concurrently in more than one thread.
1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 |
# File 'ext/oj/parser.c', line 1327
static VALUE parser_saj(VALUE self) {
if (Qundef == saj_parser) {
ojParser p = OJ_R_ALLOC(struct _ojParser);
memset(p, 0, sizeof(struct _ojParser));
buf_init(&p->key);
buf_init(&p->buf);
p->map = value_map;
oj_set_parser_saj(p);
saj_parser = TypedData_Wrap_Struct(parser_class, &oj_parser_type, p);
rb_gc_register_address(&saj_parser);
}
return saj_parser;
}
|
.usual ⇒ Object
Returns the default usual parser. Note the default usual parser can not be used concurrently in more than one thread.
1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 1315 1316 1317 |
# File 'ext/oj/parser.c', line 1304
static VALUE parser_usual(VALUE self) {
if (Qundef == usual_parser) {
ojParser p = OJ_R_ALLOC(struct _ojParser);
memset(p, 0, sizeof(struct _ojParser));
buf_init(&p->key);
buf_init(&p->buf);
p->map = value_map;
oj_set_parser_usual(p);
usual_parser = TypedData_Wrap_Struct(parser_class, &oj_parser_type, p);
rb_gc_register_address(&usual_parser);
}
return usual_parser;
}
|
.validate ⇒ Object
Returns the default validate parser.
1349 1350 1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 1361 1362 |
# File 'ext/oj/parser.c', line 1349
static VALUE parser_validate(VALUE self) {
if (Qundef == validate_parser) {
ojParser p = OJ_R_ALLOC(struct _ojParser);
memset(p, 0, sizeof(struct _ojParser));
buf_init(&p->key);
buf_init(&p->buf);
p->map = value_map;
oj_set_parser_validator(p);
validate_parser = TypedData_Wrap_Struct(parser_class, &oj_parser_type, p);
rb_gc_register_address(&validate_parser);
}
return validate_parser;
}
|
Instance Method Details
#file(filename) ⇒ Object
Parse a JSON file.
Returns the result according to the delegate of the parser.
1221 1222 1223 1224 1225 1226 1227 1228 1229 1230 1231 1232 1233 1234 1235 1236 1237 1238 1239 1240 1241 1242 1243 1244 1245 1246 1247 1248 1249 1250 1251 1252 1253 1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 |
# File 'ext/oj/parser.c', line 1221
static VALUE parser_file(VALUE self, VALUE filename) {
ojParser p;
const char *path;
int fd;
TypedData_Get_Struct(self, struct _ojParser, &oj_parser_type, p);
path = StringValuePtr(filename);
parser_reset(p);
p->start(p);
if (0 > (fd = open(path, O_RDONLY))) {
rb_raise(rb_eIOError, "error opening %s", path);
}
#if USE_THREAD_LIMIT
struct stat info;
// st_size will be 0 if not a file
if (0 == fstat(fd, &info) && USE_THREAD_LIMIT < info.st_size) {
// Use threaded version.
// TBD only if has pthreads
// TBD parse_large(p, fd);
return p->result(p);
}
#endif
byte buf[16385];
size_t size = sizeof(buf) - 1;
size_t rsize;
while (true) {
if (0 < (rsize = read(fd, buf, size))) {
buf[rsize] = '\0';
parse(p, buf);
}
if (rsize <= 0) {
if (0 != rsize) {
rb_raise(rb_eIOError, "error reading from %s", path);
}
break;
}
}
return p->result(p);
}
|
#just_one ⇒ Object
Returns the current state of the just_one [Boolean] option.
1270 1271 1272 1273 1274 1275 1276 |
# File 'ext/oj/parser.c', line 1270
static VALUE parser_just_one(VALUE self) {
ojParser p;
TypedData_Get_Struct(self, struct _ojParser, &oj_parser_type, p);
return p->just_one ? Qtrue : Qfalse;
}
|
#just_one=(value) ⇒ Object
Sets the just_one option which limits the parsing of a string or or stream to a single JSON element.
Returns the current state of the just_one [Boolean] option.
1286 1287 1288 1289 1290 1291 1292 1293 1294 |
# File 'ext/oj/parser.c', line 1286
static VALUE parser_just_one_set(VALUE self, VALUE v) {
ojParser p;
TypedData_Get_Struct(self, struct _ojParser, &oj_parser_type, p);
p->just_one = (Qtrue == v);
return p->just_one ? Qtrue : Qfalse;
}
|
#load(reader) ⇒ Object
Parse a JSON stream.
Returns the result according to the delegate of the parser.
1202 1203 1204 1205 1206 1207 1208 1209 1210 1211 1212 |
# File 'ext/oj/parser.c', line 1202
static VALUE parser_load(VALUE self, VALUE reader) {
ojParser p;
TypedData_Get_Struct(self, struct _ojParser, &oj_parser_type, p);
parser_reset(p);
p->reader = reader;
rb_rescue2(load, self, load_rescue, Qnil, rb_eEOFError, 0);
return p->result(p);
}
|
#parse(json) ⇒ Object
Parse a JSON string.
Returns the result according to the delegate of the parser.
1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 |
# File 'ext/oj/parser.c', line 1153
static VALUE parser_parse(VALUE self, VALUE json) {
ojParser p;
const byte *ptr = (const byte *)StringValuePtr(json);
TypedData_Get_Struct(self, struct _ojParser, &oj_parser_type, p);
parser_reset(p);
p->start(p);
parse(p, ptr);
validate_document_end(p);
return p->result(p);
}
|